import { ChangeDetectorRef, Component, ElementRef, Input, ViewChild } from '@angular/core';
import { Observable } from 'rxjs';
import { DateTime } from 'luxon';
import { GanttView } from './views/view';

export interface GanttItem {
  id: string;
  title: string;
  items: GanttBarElement[];
}

export interface GanttBarElement {
  title: string,
  start: number,
  duration_sec: number,
}

interface WorkTime {
  start_day_sec: number,
  duration_sec: number,
}

interface IntervalDescriptor {
  title: string;
  duration: number,
}

@Component({
  selector: 'app-gantt',
  imports: [],
  templateUrl: './gantt.component.html',
  styleUrl: './gantt.component.scss'
})
export class GanttComponent {
  @ViewChild('barContainer') _barContainer!: ElementRef;

  @Input() items: GanttItem[] = [];

  view: GanttView;

  globalBaseFontSize: number = 16;
  barContainerWidth: number = 0;

  startDate: DateTime;

  weekOpenTime: WorkTime[][] = [
    [{ start_day_sec: 8 * 3600, duration_sec: 8 * 3600 }],
    [{ start_day_sec: 8 * 3600, duration_sec: 8 * 3600 }],
    [{ start_day_sec: 8 * 3600, duration_sec: 8 * 3600 }],
    [{ start_day_sec: 8 * 3600, duration_sec: 8 * 3600 }],
    [{ start_day_sec: 8 * 3600, duration_sec: 8 * 3600 }],
    [{ start_day_sec: 8 * 3600, duration_sec: 8 * 3600 }],
    [{ start_day_sec: 8 * 3600, duration_sec: 8 * 3600 }],
    [{ start_day_sec: 8 * 3600, duration_sec: 8 * 3600 }],
  ];

  constructor(private changeDetectorRef: ChangeDetectorRef) {
    this.startDate = DateTime.now().minus({ weeks: 1 }).startOf('week').startOf('day').setLocale('de');

    this.view = new GanttView();

    let htmlRoot: HTMLElement = <HTMLElement>document.getElementsByTagName("html")[0];
  }

  ngAfterViewInit(): void {
    // get base font size
    let fontSize = window.getComputedStyle(this._barContainer.nativeElement).getPropertyValue('font-size');
    this.globalBaseFontSize = parseInt(fontSize.slice(0, fontSize.length - 2));
    this.view.globalBaseFontSize = parseInt(fontSize.slice(0, fontSize.length - 2));

    // observe resizing of the bar container
    new Observable((observer) => {
      const resizeObserver = new ResizeObserver(() => {
        observer.next();
      });
      resizeObserver.observe(this._barContainer.nativeElement);
    }).subscribe((change) => {
      this.barContainerWidth = this._barContainer.nativeElement.offsetWidth;
      this.view.width = this._barContainer.nativeElement.offsetWidth;

      this.changeDetectorRef.detectChanges();
    });
  }

  changeViewType(value: string): void {
    this.view.changeView(value)
  }

  getDate(day: number): string {
    return this.startDate.plus({ days: day }).toFormat('dd.MM');
  }

  getWeekday(day: number): string {
    return this.startDate.plus({ days: day }).toFormat('EEE');
  }

  getDayDuration(day: number): number {
    if (day === 0) {
      return Math.abs(this.startDate.diff(this.startDate.endOf('day')).as('seconds'));
    } else {
      return 24 * 3600;
    }
  }

  getDayWorkTime(day: number): WorkTime[] {
    return this.weekOpenTime[day];
  }

  durationToWidth(seconds: number, opts?: { offsetCorrection?: number }): string {
    let correction = opts?.offsetCorrection ?? 0;
    return `${this.pixelPerSec * seconds + correction}px`;
  }

  startToOffset(date: DateTime | number, opts?: { dstCorrection?: boolean, offsetCorrection?: number }): string {
    // generate date from timestamp if necessary
    if (typeof date === 'number')
      date = DateTime.fromSeconds(date);
    // calculate offset 
    let offset = this.dateDiff(date, this.startDate, opts) * this.pixelPerSec;
    let correction = opts?.offsetCorrection ?? 0;

    return `${offset + correction}px`;
  }

  dateDiff(date1: DateTime, date2: DateTime, opts?: { dstCorrection?: boolean }): number {
    // calculate offset 
    let diff = date1.diff(date2, 'seconds').as('seconds');

    //modify if DST (Daylight Saving Time) has changed // TODO Check if DST Switch is problematic
    if (opts?.dstCorrection) {
      if (date1.isInDST && !date2.isInDST) {
        diff += 3600;
      } else if (date1.isInDST && !date2.isInDST) {
        diff -= 3600;
      }
    }

    return diff;
  }

  firstDayIntervalShift(day: number): string {
    return `${this.getDayDuration(day) / this.segmentBaseSeconds * 100 - 100}%`;
  }

  moveBack(): void {
    this.startDate = this.startDate.minus({ days: 3.5 });
  }
  moveNext(): void {
    this.startDate = this.startDate.plus({ days: 3.5 });
  }

  get segmentBaseSeconds(): number {
    return 3600 * 24;
  }

  get pixelPerSec(): number {
    const SecondsPerWeek = 3600 * 24 * 7;
    return this.barContainerWidth / SecondsPerWeek;
  }

  get month(): string {
    return this.startDate.toFormat('MMMM yyyy')

  }

  get week(): string {
    return this.startDate.toFormat('nn')
  }

  // TODO do not recalculate for every element but only once when window is resized or forward or backword is pressed
  get hourArray(): number[] {
    let base = 3600 * 24;
    let intervals = base * this.pixelPerSec / (2 * 0.8 * this.globalBaseFontSize);

    // TODO make calculation more sane!!!
    let test = Math.floor(24 / Math.ceil((2 * 0.8 * this.globalBaseFontSize) / (base * this.pixelPerSec / 24)));

    return Array.from({ length: test }, (_, i) => i * 24 / test)
    // return Array.from({ length: 23 }, (_, i) => i + 1)
  }

  get intervalArray(): IntervalDescriptor[] {
    let start = this.startDate.get('weekNumber');
    let end = this.startDate.plus({ week: 1 }).minus({ seconds: 10 }).get('weekNumber');
    let arr: IntervalDescriptor[] = [];

    for (let i = start; i <= end; i++) {
      if (i === start && start !== end) {
        arr.push({
          title: `KW ${i}`,
          duration: this.dateDiff(this.startDate.endOf('week'), this.startDate, { dstCorrection: true }) //this.startDate.endOf('week').diff(this.startDate).as('seconds')
        })
      } else if (i === end) {
        arr.push({
          title: `KW ${i}`,
          duration: 0 // last element grows automatically
        })
      } else { }
    }

    return arr;
  }
}
